#  /**************************************************
#   * Copyright (c) 2024 HSD or HSD Ltd
#   *
#   * This program is free software: you can redistribute it and/or modify
#   * it under the terms of the GNU General Public License as published by
#   * the Free Software Foundation, either version 3 of the License, or
#   * (at your option) any later version.
#   *
#   * This program is distributed in the hope that it will be useful,
#   * but WITHOUT ANY WARRANTY; without even the implied warranty of
#   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   * GNU General Public License for more details.
#   *
#   * You should have received a copy of the GNU General Public License
#   * along with this program. If not, see <https://www.gnu.org/licenses/>.
#   **************************************************/
import datetime
import os
import google.auth
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
import time

from helpers.file_properties import file_hashes


# Path to your OAuth 2.0 credentials JSON file (downloaded from Google Cloud Console)
credentials_path = 'credentials.json'
token_path = 'token.json'  # This file will store your tokens

# Setup authentication using OAuth 2.0 credentials (must have valid credentials)
scopes = ["https://www.googleapis.com/auth/youtube.upload"]


def get_authenticated_service():
    creds = None

    # Check if token.json exists and load the credentials from it
    if os.path.exists(token_path):
        creds = Credentials.from_authorized_user_file(token_path, scopes)

    # If credentials are not available or are invalid, run the authorization flow
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            # Refresh the token if it has expired
            creds.refresh(google.auth.transport.requests.Request())
        else:
            # Otherwise, start a new OAuth flow
            flow = InstalledAppFlow.from_client_secrets_file(credentials_path, scopes)
            creds = flow.run_local_server(port=0)

        # Save the credentials for the next run
        with open(token_path, 'w') as token_file:
            token_file.write(creds.to_json())

    return build('youtube', 'v3', credentials=creds)


def upload_scheduled_video_to_youtube(video_file_path, title, description, tags, category_id, publish_time,
                                      thumbnail_file_path=None):
    # Build the YouTube API client
    youtube = get_authenticated_service()

    # Define video metadata
    body = {
        "snippet": {
            "title": title,
            "description": description,
            "tags": tags,
            "categoryId": category_id,  # Category of the video, e.g., 22 (People & Blogs)
            "publishAt": publish_time.isoformat() + 'Z',  # The scheduled publish time (ISO 8601 format)
        },
        "status": {
            "privacyStatus": "private",  # Initially upload as private
            "publishAt": publish_time.isoformat() + 'Z',  # Schedule publish date/time
        }
    }

    # Media upload
    media = MediaFileUpload(video_file_path, chunksize=-1, resumable=True)

    # Upload the video
    upload_response = youtube.videos().insert(
        part="snippet,status",
        body=body,
        media_body=media
    ).execute()

    print(f"Video uploaded with ID: {upload_response['id']}")

    # Optionally, upload a custom thumbnail
    if thumbnail_file_path:
        youtube.thumbnails().set(
            videoId=upload_response['id'],
            media_body=MediaFileUpload(thumbnail_file_path)
        ).execute()

    print("Video is scheduled to be published at", publish_time)


# Example usage:
# upload_scheduled_video_to_youtube(
#     video_file_path='path/to/video.mp4',
#     title='My Scheduled Video',
#     description='This is a test video.',
#     tags=['python', 'youtube', 'upload'],
#     category_id='22',  # People & Blogs category
#     publish_time=datetime.datetime(2023, 10, 10, 12, 0),  # UTC time
#     thumbnail_file_path='path/to/thumbnail.jpg'
# )

#
if __name__ == '__main__':
    start_time = time.time()
    folder_path = '6702e02f-a6ed-8886-2415-f0d9bbe12412.mp4'

    upload_scheduled_video_to_youtube(
        video_file_path=folder_path,
        title='Smart',
        description='Smart',
        tags=['Funny videos', 'Pranks', 'Epic fails', 'Funny moments', 'Laugh-out-loud', 'Best pranks', 'Viral pranks', 'Humor', 'Hilarious reactions'],
        category_id='22',  # People & Blogs category
        publish_time=datetime.datetime(2024, 10, 11, 12, 0),  # UTC time 2024.10.11 12:00 -> UK time 2024.10.11 13:00
    )
    print("--- %.10f seconds ---" % (time.time() - start_time))
    print("done")
